@@ -53,7 +53,18 @@ bool_int checkinlist;
     }
     UNLINK(tmpname);			/* be nice to the world */
 
-    while (fgets(buf,LBUFLEN,actfp) != Nullch) {
+    for (;;) {
+#ifdef USE_NNTP
+	if (!actfp) {
+	    if (NNTP_LIST_END(ser_line))
+		break;
+	    strcpy(buf, ser_line);
+	    nntp_gets(ser_line, sizeof ser_line);
+	}
+	else
+#endif
+	if (fgets(buf,LBUFLEN,actfp) == Nullch)
+	    break;
 	/* Check if they want to break out of the new newsgroups search */
 	if (int_count) {
 	    int_count = 0;
@@ -226,6 +237,12 @@ find_new_groups()
     long oldactsiz = lastactsiz;
     NG_NUM oldnext = nextrcline;	/* remember # lines in newsrc */
 
+#ifdef USE_NNTP
+    if (!actfp) {
+	return 0;
+    }
+#endif
+
     fstat(fileno(actfp),&filestat);	/* did active file grow? */
 
     if (filestat.st_size == lastactsiz)
@@ -290,13 +307,28 @@ scanactive()
 {
     NG_NUM oldnext = nextrcline;	/* remember # lines in newsrc */
 
-    fseek(actfp,0L,0);
-    newlist(TRUE,TRUE);
+    if (actfp) {
+	fseek(actfp,0L,0);
+	newlist(TRUE,TRUE);
+    }
+#ifdef USE_NNTP
+    else {
+	if (maxngtodo != 1) {
+	    strcpy(buf, "*");
+	}
+	else {
+	    sprintf(buf,"*%s*", ngtodo[0]);
+	}
+	if (nntp_list("active", buf, strlen(buf)) == 1) {
+	    newlist(TRUE,TRUE);
+	}
+    }
+#endif
     if (nextrcline != oldnext) {	/* did we add any new groups? */
 	return TRUE;
     }
     return FALSE;
 }
 
-#endif
+#endif /* FINDNEWNG */
 
