@@ -31,6 +31,8 @@
 #include "term.h"
 #include "sw.h"
 #include "util.h"
+#include "util2.h"
+#include "nntp.h"
 #include "backpage.h"
 #include "intrp.h"
 #include "rthread.h"
@@ -50,20 +52,14 @@
 
 bool special = FALSE;		/* is next page special length? */
 int slines = 0;			/* how long to make page when special */
-ART_LINE highlight = -1;	/* next line to be highlighted */
 char *restart = Nullch;		/* if nonzero, the place where last */
 				/* line left off on line split */
 char *blinebeg;			/* where in buffer current line began */
 ART_POS alinebeg;		/* where in file current line began */
 
 #ifdef INNERSEARCH
-ART_POS innersearch = 0;	/* artpos of end of line we found */
-				/* for 'g' command */
-ART_LINE isrchline = 0;			/* last line to display */
-bool hide_everything = FALSE;
-				/* if set, do not write page now, */
-				/* but refresh when done with page */
-COMPEX gcompex;				/* in article search pattern */
+ART_LINE isrchline = 0;		/* last line to display */
+COMPEX gcompex;			/* in article search pattern */
 #endif
 
 bool firstpage;			/* is this the 1st page of article? */
@@ -108,6 +104,9 @@ display_mime()
 	if (*buf == 'y') {
 	    putchar('\n');
 	    fflush(stdout);
+#ifdef USE_NNTP
+	    nntp_finishbody(FB_OUTPUT);
+#endif
 	    termlib_reset();
 	    resetty();
 	    interp(cmd_buf,(sizeof cmd_buf),getval("MIMESHOW",MIMESHOW));
@@ -126,7 +125,6 @@ int
 do_article()
 {
     register char *s;
-    ART_POS artsize;			/* size in bytes of article */
     bool hide_this_line = FALSE;	/* hidden header line? */
     ART_LINE linenum;	/* line # on page, 1 origin */
 #ifdef ULSMARTS
@@ -149,11 +147,13 @@ do_article()
     mime_article = FALSE;
 #endif
 
+#ifndef USE_NNTP
     if (fstat(fileno(artfp),&filestat)) /* get article file stats */
 	return DA_CLEAN;
     if (!S_ISREG(filestat.st_mode))
 	return DA_NORM;
     artsize = filestat.st_size;		/* from that get article size */
+#endif
 #ifdef CHARSUBST
     sprintf(prompt_buf,
 	"%%sEnd of article %ld (of %ld) %%s-- what next? [%%s]",
@@ -167,7 +167,7 @@ do_article()
     int_count = 0;		/* interrupt count is 0 */
     if ((firstpage = (topline < 0)) != 0) {
 	parseheader(art);
-	fseek(artfp,htype[PAST_HEADER].ht_minpos,0);
+	seekart(htype[PAST_HEADER].ht_minpos);
     }
     for (;;) {			/* for each page */
 	if (ThreadedGroup && max_tree_lines)
@@ -182,9 +182,9 @@ do_article()
 		artpos = (ART_POS)0;
 	    if (artpos < htype[PAST_HEADER].ht_minpos) {
 		in_header = SOME_LINE;
-		fseek(artfp,htype[PAST_HEADER].ht_minpos,0);
+		seekart(htype[PAST_HEADER].ht_minpos);
 	    } else
-		fseek(artfp,artpos,0);
+		seekart(artpos);
 	    do_fseek = FALSE;
 	    restart = Nullch;
 	}
@@ -234,6 +234,9 @@ do_article()
 		int_count = 0;	/* reset interrupt count */
 		mode = oldmode;
 		special = FALSE;
+#ifdef USE_NNTP
+		nntp_finishbody(FB_OUTPUT);
+#endif
 		return DA_NORM;	/* skip out of loops */
 	    }
 	    if (restart) {		/* did not finish last line? */
@@ -246,11 +249,8 @@ do_article()
 		art_buf[bufptr-headbuf-artpos] = '\0';
 		bufptr = art_buf;
 	    } else {
-		if (fgets(art_buf,LBUFLEN,artfp)==Nullch) {
-					/* if all done */
-		    mode = oldmode;
-		    special = FALSE;
-		    return DA_NORM;	/* skip out of loops */
+		if (readart(art_buf,LBUFLEN) == Nullch) {
+		    break;
 		}
 		bufptr = art_buf;	/* so start at beginning */
 		art_buf[LBUFLEN-1] = '\0';
@@ -268,10 +268,10 @@ do_article()
 	    } else if (notesfiles && do_hiding &&
 	      bufptr == art_buf && *art_buf == '#' &&
 	      isupper(art_buf[1]) && art_buf[2] == ':' ) {
-		fgets(art_buf,sizeof(art_buf),artfp);
+		readart(art_buf,sizeof(art_buf));
 		if (index(art_buf,'!') != Nullch)
-		    fgets(art_buf,sizeof(art_buf),artfp);
-		htype[PAST_HEADER].ht_minpos = ftell(artfp);
+		    readart(art_buf,sizeof(art_buf));
+		htype[PAST_HEADER].ht_minpos = tellart();
 					/* exclude notesfiles droppings */
 		hide_this_line = TRUE;	/* and do not print either */
 		notesfiles = FALSE;
@@ -348,8 +348,7 @@ do_article()
 		if (mime_article && do_hiding && !tried_display_mime) {
 		    if (display_mime() == 0)
 			return DA_NORM;
-		    else
-			tried_display_mime = TRUE;
+		    tried_display_mime = TRUE;
 		}
 #endif
 #ifdef CLEAREOL
@@ -391,15 +390,13 @@ do_article()
 #ifdef ULSMARTS
 			if (*bufptr == '_') {
 			    if (bufptr[1] == '\b') {
-				if (!under_lining && highlight!=artline
 #ifdef INNERSEARCH
-				    && outputok
+				if (outputok)
 #endif
-				    ) {
+				if (!under_lining && highlight != artline) {
 				    under_lining++;
 				    if (UG) {
-					if (bufptr != buf &&
-					  bufptr[-1] == ' ') {
+					if (bufptr != buf && bufptr[-1]==' ') {
 					    outpos--;
 					    backspace();
 					}
@@ -421,29 +418,39 @@ do_article()
 			    }
 			}
 #endif
-#ifdef INNERSEARCH
-			if (outputok)
-#endif
-			{
 #ifdef ROTATION
-			    if (rotate && !in_header
-			      && isalpha(*bufptr)) {
+			if (rotate && !in_header && isalpha(*bufptr)) {
+# ifdef INNERSEARCH
+			    if (outputok)
+# endif
 				if ((*bufptr & 31) <= 13)
 				    putchar(*bufptr+13);
 				else
 				    putchar(*bufptr-13);
-				outpos++;
-			    }
-			    else
+			    outpos++;
+			}
+			else
 #endif
-			    {
+			{
 #ifdef CHARSUBST
-				outpos += putsubstchar(*bufptr);
-#else
-				putchar(*bufptr);
-				outpos++;
-#endif
+			    register int i;
+# ifdef INNERSEARCH
+			    i = putsubstchar(*bufptr, COLS - outpos, outputok);
+# else
+			    i = putsubstchar(*bufptr, COLS - outpos, TRUE);
+# endif
+			    if (i < 0) {
+				outpos += -i - 1;
+				break;
 			    }
+			    outpos += i;
+#else
+# ifdef INNERSEARCH
+			    if (outputok)
+# endif
+				putchar(*bufptr);
+			    outpos++;
+#endif /* CHARSUBST */
 			}
 			if (*UC && ((highlight==artline && marking == STANDOUT)
 #ifdef ULSMARTS
@@ -491,6 +498,8 @@ do_article()
 			outpos += 8 - outpos % 8;
 		    }
 		    else if (*bufptr == '\f') {	/* form feed? */
+			if (outpos+2 > COLS)
+			    break;
 #ifdef INNERSEARCH
 			if (outputok)
 #endif
@@ -502,15 +511,20 @@ do_article()
 			outpos += 2;
 		    }
 		    else {		/* other control char */
+			if (dont_filter_control) {
 #ifdef INNERSEARCH
-			if (outputok)
+			    if (outputok)
 #endif
-			{
-			    if (dont_filter_control) {
 				putchar(*bufptr);
-				outpos++;
-			    }
-			    else {
+			    outpos++;
+			}
+			else if (*bufptr != '\r' || bufptr[1] != '\n') {
+			    if (outpos+2 > COLS)
+				break;
+#ifdef INNERSEARCH
+			    if (outputok)
+#endif
+			    {
 				putchar('^');
 				if (highlight == artline && *UC
 				 && marking == STANDOUT) {
@@ -522,19 +536,18 @@ do_article()
 				}
 				else
 				    putchar(*bufptr+64);
-				outpos += 2;
 			    }
+			    outpos += 2;
 			}
 			bufptr++;
 		    }
 		    
 		} /* end of column loop */
 
-		if (outpos < 1000) {/* did line overflow? */
-		    restart = bufptr;
-				    /* restart here next time */
-		    if (!AM || XN) {/* no automatic margins on tty? */
-#ifdef INNERSEARCH		    /* then move it down ourselves */
+		if (outpos < 1000) {  /* did line overflow? */
+		    restart = bufptr; /* restart here next time */
+		    if (!AM || XN || outpos < COLS) {
+#ifdef INNERSEARCH
 			if (outputok)
 #endif
 			    putchar('\n') FLUSH;
@@ -571,15 +584,15 @@ do_article()
 	    else if (in_header)
 		artpos = index(headbuf+artpos, '\n') - headbuf + 1;
 	    else		/* no, ftell will do */
-		artpos = ftell(artfp);
+		artpos = tellart();
 	    vwtary(artline,artpos);	/* remember pos in file */
 	} /* end of line loop */
 
 #ifdef INNERSEARCH
 	innersearch = 0;
 	if (hide_everything) {
-	    hide_everything = FALSE;
-	    *buf = Ctl('l');
+	    *buf = hide_everything;
+	    hide_everything = 0;
 	    goto fake_command;
 	}
 #endif
@@ -589,9 +602,14 @@ do_article()
 
 	special = FALSE;	/* end of page, so reset page length */
 	firstpage = FALSE;	/* and say it is not 1st time thru */
+	highlight = -1;
 
 	/* extra loop bombout */
 
+#ifdef USE_NNTP
+	artsize = nntp_artsize();
+recheck_pager:
+#endif
 	if (artpos == artsize) {/* did we just now reach EOF? */
 	    mode = oldmode;
 	    return DA_NORM;	/* avoid --MORE--(100%) */
@@ -602,11 +620,16 @@ do_article()
 reask_pager:		    
 	unflush_output();	/* disable any ^O in effect */
 	standout();		/* enter standout mode */
+#ifdef USE_NNTP
+	if (artsize == -1)
+	    strcpy(buf,"?");
+	else
+#endif
+	    sprintf(buf,"%ld",(long)(artpos*100/artsize));
 #ifdef CHARSUBST
-	printf("%s--MORE--(%ld%%)",current_charsubst(),
-	       (long)(artpos*100/artsize));
+	printf("%s--MORE--(%s%%)",current_charsubst(),buf);
 #else
-	printf("--MORE--(%ld%%)",(long)(artpos*100/artsize));
+	printf("--MORE--(%s%%)",buf);
 #endif
 	un_standout();	/* leave standout mode */
 #ifdef CLEAREOL
@@ -629,6 +652,12 @@ reask_pager:
 	    checkpoint_rc();	/* update .newsrc */
 	}
 	cache_until_key();
+#ifdef USE_NNTP
+	if (artsize == -1 && (artsize = nntp_artsize()) != -1) {
+	    carriage_return();
+	    goto recheck_pager;
+	}
+#endif
 	mode = 'p';
 	getcmd(buf);
 	if (errno) {
@@ -691,8 +720,8 @@ page_switch()
 		pos = -pos;
 	    if (pos < htype[PAST_HEADER].ht_minpos)
 		break;
-	    fseek(artfp,(long)pos,0);
-	    if (fgets(s = buf+1, sizeof buf - 1, artfp) == Nullch) {
+	    seekart((long)pos);
+	    if (readart(s = buf+1, sizeof buf - 1) == Nullch) {
 		s = blinebeg;
 		break;
 	    }
@@ -741,20 +770,21 @@ page_switch()
 	}
 	if (start_where < htype[PAST_HEADER].ht_minpos)
 	    start_where = htype[PAST_HEADER].ht_minpos;
-	fseek(artfp,(long)start_where,0);
+	seekart((long)start_where);
+	innerlight = 0;
 	innersearch = 0; /* assume not found */
-	while (fgets(buf, sizeof buf, artfp) != Nullch) {
+	while (readart(buf, sizeof buf) != Nullch) {
 #ifdef DEBUG
 	    if (debug & DEB_INNERSRCH)
 		printf("Test %s",buf) FLUSH;
 #endif
 	    if (execute(&gcompex,buf) != Nullch) {
-		innersearch = ftell(artfp);
+		innersearch = tellart();
 		break;
 	    }
 	}
 	if (!innersearch) {
-	    fseek(artfp,artpos,0);
+	    seekart(artpos);
 	    fputs("(Not found)",stdout) FLUSH;
 	    return PS_ASK;
 	}
@@ -783,7 +813,7 @@ page_switch()
 	}
 	else {				/* who knows how many lines it is? */
 	    do_fseek = TRUE;
-	    hide_everything = TRUE;
+	    hide_everything = Ctl('l');
 	}
 	return PS_NORM;
     }
@@ -807,7 +837,7 @@ page_switch()
 #ifdef DEBUG
 	if (debug & DEB_INNERSRCH) {
 	    printf("Topline = %d",topline) FLUSH;
-	    gets(buf);
+	    fgets(buf, sizeof buf, stdin);
 	}
 #endif
 	clear();
@@ -818,6 +848,18 @@ page_switch()
 	    artline = 0;
 	firstpage = (topline < 0);
 	return PS_NORM;
+#ifdef INNERSEARCH
+    case Ctl('e'):
+#ifdef USE_NNTP
+	nntp_finishbody(FB_OUTPUT);
+	artsize = nntp_artsize();
+#endif
+	innerlight = artline - 1;
+	innersearch = artsize;
+	gline = 0;
+	hide_everything = 'b';
+	return PS_NORM;
+#endif
     case 'B':		/* one line up */
 	if (topline < 0)
 	    break;
@@ -830,14 +872,14 @@ page_switch()
 	    if (pos < 0)
 		pos = -pos;
 	    if (pos >= htype[PAST_HEADER].ht_minpos) {
-		fseek(artfp,(long)pos,0);
-		if (fgets(s = buf, sizeof buf, artfp) != Nullch) {
+		seekart((long)pos);
+		if (readart(s = buf, sizeof buf) != Nullch) {
 		    fputs(buf, stdout) FLUSH;
 		    topline--;
 		    artpos = vrdary(--artline);
 		    if (artpos < 0)
 			artpos = -artpos;
-		    fseek(artfp,artpos,0);
+		    seekart(artpos);
 		    goto_line(1, LINES-1);
 		    erase_eol();
 		    carriage_return();
@@ -922,7 +964,8 @@ page_switch()
     case '[':	case ']':
     case '{':	case '}':
     case '(':   case ')':
-    case '+':   case ':':
+    case ':':
+    case '+':
     case '#':
     case '$':
     case '&':
@@ -960,14 +1003,18 @@ leave_pager:
 #endif
 	reread = FALSE;
 	do_hiding = TRUE;
+#ifdef USE_NNTP
+	nntp_finishbody(FB_OUTPUT);
+#endif
 	if (index("nNpP\016\020",*buf) == Nullch &&
 	  index("wWsSe:!&|/?123456789.",*buf) != Nullch) {
 	    setdfltcmd();
 	    standout();		/* enter standout mode */
+	    interp(cmd_buf, sizeof cmd_buf, mailcall);
 #ifdef CHARSUBST
-	    printf(prompt,mailcall,current_charsubst(),dfltcmd);
+	    printf(prompt,cmd_buf,current_charsubst(),dfltcmd);
 #else
-	    printf(prompt,mailcall,dfltcmd);
+	    printf(prompt,cmd_buf,dfltcmd);
 #endif
 			    /* print prompt, whatever it is */
 	    un_standout();	/* leave standout mode */
@@ -1029,6 +1076,9 @@ leave_pager:
 	return PS_NORM;
     case 'q':	/* quit this article? */
 	do_hiding = TRUE;
+#ifdef USE_NNTP
+	nntp_finishbody(FB_OUTPUT);
+#endif
 	return PS_TOEND;
     default:
 	fputs(hforhelp,stdout) FLUSH;
@@ -1052,14 +1102,17 @@ innermore()
     }
     if (artpos == innersearch) {	/* just got onto page? */
 	isrchline = artline;		/* remember first line after */
-	highlight = artline - 1;
+	if (innerlight)
+	    highlight = innerlight;
+	else
+	    highlight = artline - 1;
 #ifdef DEBUG
 	if (debug & DEB_INNERSRCH)
 	    printf("There it is %ld = %ld, %d @ %d\n",(long)artpos,
 		(long)innersearch,hide_everything,highlight) FLUSH;
 #endif
 	if (hide_everything) {		/* forced refresh? */
-	    topline = highlight - gline;
+	    topline = artline - gline - 1;
 	    if (topline < -1)
 		topline = -1;
 	    return FALSE;		/* let refresh do it all */
