@@ -30,6 +30,7 @@
 #include "bits.h"
 #include "rcstuff.h"
 #include "hash.h"
+#include "nntp.h"
 #include "rthread.h"
 #include "rt-ov.h"
 #include "rt-page.h"
@@ -37,6 +38,7 @@
 #include "rt-select.h"
 #include "rt-util.h"
 #include "util.h"
+#include "util2.h"
 
 #ifdef PENDING
 #   ifdef ARTSEARCH
@@ -85,7 +87,7 @@ build_cache()
     cached_ng = ng;
     cached_absfirst = absfirst;
     cached_time = time((time_t*)NULL);
-    cached_cnt = lastart-absfirst+2 + 5;
+    cached_cnt = lastart-absfirst+2 + 10;
     article_list = (ARTICLE*)
 	safemalloc((MEM_SIZE)(cached_cnt * sizeof (ARTICLE)));
     bzero((char*)article_list, cached_cnt * sizeof (ARTICLE));
@@ -107,8 +109,8 @@ build_cache()
     thread_open();
 }
 
-#define FIXPTR(p) (!(p) || (p)<old_list || ((p)-old_list) > (lastart-absfirst)\
-		   ? (p) : (article_list + ((p)-old_list)))
+#define FIXPTR(p) if ((p) && (p) >= oldlist_start && (p) <= oldlist_end)\
+		      (p) = (article_list + ((p) - oldlist_start)); else
 
 void
 grow_cache(newlast)
@@ -117,33 +119,36 @@ ART_NUM newlast;
     ART_NUM new_cnt = newlast-absfirst+2;
 
     if (new_cnt > cached_cnt) {
-	ARTICLE *old_list = article_list;
-	new_cnt += 5;
+	ARTICLE *oldlist_start = article_list;
+	ARTICLE *oldlist_end = oldlist_start + cached_cnt;
+	new_cnt += 10;
 	article_list = (ARTICLE*)saferealloc((char*)article_list,
 		(MEM_SIZE)(new_cnt * sizeof (ARTICLE)));
 	bzero((char*)(article_list+cached_cnt),
 		(new_cnt-cached_cnt) * sizeof (ARTICLE));
-	if (article_list != old_list) {
+	if (article_list != oldlist_start) {
 	    register ARTICLE *ap;
 	    register SUBJECT *sp;
 	    for (sp = first_subject; sp; sp = sp->next) {
-		sp->thread = FIXPTR(sp->thread);
-		sp->articles = FIXPTR(sp->articles);
+		FIXPTR(sp->thread);
+		FIXPTR(sp->articles);
 		if (sp->thread) {
 		    for (ap = sp->thread; ap; ap = bump_art(ap)) {
-			ap->child1 = FIXPTR(ap->child1);
-			ap->parent = FIXPTR(ap->parent);
-			ap->sibling = FIXPTR(ap->sibling);
-			ap->subj_next = FIXPTR(ap->subj_next);
+			FIXPTR(ap->child1);
+			FIXPTR(ap->parent);
+			FIXPTR(ap->sibling);
+			FIXPTR(ap->subj_next);
 		    }
 		} else {
-		    for (ap = sp->articles; ap; )
-			ap = ap->subj_next = FIXPTR(ap->subj_next);
+		    for (ap = sp->articles; ap; ) {
+			FIXPTR(ap->subj_next);
+			ap = ap->subj_next;
+		    }
 		}
 	    }
-	    artp = FIXPTR(artp);
-	    curr_artp = FIXPTR(curr_artp);
-	    recent_artp = FIXPTR(recent_artp);
+	    FIXPTR(artp);
+	    FIXPTR(curr_artp);
+	    FIXPTR(recent_artp);
 	}
 	cached_cnt = new_cnt;
     }
@@ -304,23 +309,23 @@ register ARTICLE *ap;
 		if (instr(h,phostname,FALSE)) {
 		    switch (auto_select_postings) {
 		    case '.':
-			select_subthread(ap,AF_AUTOSELECT);
+			select_subthread(ap,AUTO_SELECT);
 			break;
 		    case '+':
-			select_arts_thread(ap,AF_AUTOSELECT);
+			select_arts_thread(ap,AUTO_SELECT);
 			break;
 		    case 'p':
 			if (ap->parent)
-			    select_subthread(ap->parent,AF_AUTOSELECT);
+			    select_subthread(ap->parent,AUTO_SELECT);
 			else
-			    select_subthread(ap,AF_AUTOSELECT);
+			    select_subthread(ap,AUTO_SELECT);
 			break;
 		    }
 		} else {
 #ifdef REPLYTO_POSTER_CHECKING
 		    char *reply_buf = fetchlines(article_num(ap),REPLY_LINE);
 		    if (instr(reply_buf,loginName,TRUE))
-			select_subthread(ap,AF_AUTOSELECT);
+			select_subthread(ap,AUTO_SELECT);
 		    free(reply_buf);
 #endif
 		}
@@ -466,8 +471,8 @@ register int size;
     newsubj = safemalloc(size + 4 + 1);
     strcpy(newsubj, "Re: ");
     for (t = newsubj + 4, f = subj_start, i = size; i--; ) {
-	if (*f == ' ' || *f == '\t') {
-	    while (i && (*++f == ' ' || *f == '\t'))
+	if ((unsigned char)*f <= ' ') {
+	    while (i && ((unsigned char)*++f <= ' '))
 		i--, size--;
 	    *t++ = ' ';
 	} else if (*f != '\n')
@@ -515,6 +520,7 @@ register ARTICLE *ap;
 register int which_line;
 register char *s;		/* already allocated, ready to save */
 {
+    char *cp;
     /* SUBJ_LINE is handled specially above */
     switch (which_line) {
     case FROM_LINE:
@@ -537,12 +543,12 @@ register char *s;		/* already allocated, ready to save */
     case XREF_LINE:
 	if (ap->xrefs && ap->xrefs != nullstr)
 	    free(ap->xrefs);
-# ifdef USE_NNTP
-	if (strEQ(s, "(none)")) {
+	/* Exclude an xref for just this group or "(none)". */
+	cp = index(s, ':');
+	if (!cp || !index(cp+1, ':')) {
 	    free(s);
 	    s = nullstr;
 	}
-# endif
 	ap->xrefs = s;
 	break;
 #endif
@@ -607,7 +613,7 @@ look_ahead()
 	if (debug & DEB_SEARCH_AHEAD) {
 	    fputs("(hit CR)",stdout);
 	    fflush(stdout);
-	    gets(buf+128);
+	    fgets(buf+128, sizeof buf-128, stdin);
 	    printf("\npattern = %s\n",pattern);
 	}
 #endif
@@ -664,19 +670,26 @@ cache_until_key()
     untrim_cache = TRUE;
     sentinel_artp = curr_artp;
 
+#ifdef USE_NNTP
+    if (nntp_finishbody(FB_BACKGROUND))
+	return;
+#endif
+
     /* Prioritize our caching based on what mode we're in */
     if (mode == 't') {
 	if (cache_subjects())
 	    if (cache_xrefs())
-		if (ThreadedGroup)
-		    cache_all_arts();
-		else
-		    cache_unread_arts();
+		if (chase_xrefs(TRUE))
+		    if (ThreadedGroup)
+			cache_all_arts();
+		    else
+			cache_unread_arts();
     } else {
 	if (!ThreadedGroup || cache_all_arts())
 	    if (cache_subjects())
 		if (cache_unread_arts())
-		    cache_xrefs();
+		    if (cache_xrefs())
+			chase_xrefs(TRUE);
     }
 
     setspin(SPIN_OFF);
