@@ -18,7 +18,7 @@
 EXT char *charsets INIT("patm");
 EXT char *charsubst;
 
-int  putsubstchar _((int));
+int putsubstchar _((int, int, bool_int));
 char *current_charsubst _((void));
 void strcharsubst _((char*,char*));
 
