@@ -35,4 +35,8 @@ Signal_t stop_catcher _((int));
 #endif
 
 void	final_init _((void));
-void	finalize _((int));
+void	finalize _((int))
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR >= 5)
+  __attribute__((noreturn))
+#endif
+  ;
