@@ -195,8 +195,10 @@ register int keylen;
     register HASHENT **hepp;
     register unsigned size; 
 
-    if (BADTBL(tbl))
-	fatal_error("Hash table is invalid.");
+    if (BADTBL(tbl)) {
+	fputs("Hash table is invalid.",stderr);
+	finalize(1);
+    }
     size = tbl->ht_size;
     hepp = &tbl->ht_addr[hash(key,keylen) % size];
     for (hp = *hepp; hp != NULL; prevhp = hp, hp = hp->he_next) {
