@@ -16,6 +16,7 @@
 #include "EXTERN.h"
 #include "common.h"
 #include "util.h"
+#include "util2.h"
 #include "search.h"
 #include "cache.h"
 #include "bits.h"
@@ -101,12 +102,13 @@ char *tcbuf;
     /* get login name */
 
     loginName = getenv("USER");
-    if (loginName == Nullch)
+    if (loginName == Nullch) {
 	loginName = getenv("LOGNAME");
 #ifdef GETLOGIN
-    if (loginName == Nullch)
-	loginName = savestr(getlogin());
+	if (loginName == Nullch)
+	    loginName = savestr(getlogin());
 #endif
+    }
 
     spool = savestr(filexp(NEWSSPOOL));	/* usually /usr/spool/news */
     threaddir = filexp(THREAD_DIR);
@@ -454,7 +456,7 @@ char *stoppers;
     bool upper = FALSE;
     bool lastcomp = FALSE;
     bool re_quote = FALSE;
-    bool tick_quote = FALSE;
+    int tick_quote = 0;
     bool address_parse = FALSE;
     bool comment_parse = FALSE;
     bool proc_sprintf = FALSE;
@@ -469,7 +471,7 @@ char *stoppers;
 	    upper = FALSE;
 	    lastcomp = FALSE;
 	    re_quote = FALSE;
-	    tick_quote = FALSE;
+	    tick_quote = 0;
 	    address_parse = FALSE;
 	    comment_parse = FALSE;
 	    proc_sprintf = FALSE;
@@ -485,7 +487,7 @@ char *stoppers;
 		    re_quote = TRUE;
 		    break;
 		case '\'':
-		    tick_quote = TRUE;
+		    tick_quote++;
 		    break;
 		case '>':
 		    address_parse = TRUE;
@@ -546,11 +548,15 @@ char *stoppers;
 		    s = getval(scrbuf,s);
 		    break;
 		case '[':
-		    pattern = cpytill(scrbuf,pattern+1,']');
-		    i = set_line_type(scrbuf,scrbuf+strlen(scrbuf));
-		    if (line_buf)
-			free(line_buf);
-		    s = line_buf = fetchlines(art,i);
+		    if (in_ng) {
+			pattern = cpytill(scrbuf,pattern+1,']');
+			i = set_line_type(scrbuf,scrbuf+strlen(scrbuf));
+			if (line_buf)
+			    free(line_buf);
+			s = line_buf = fetchlines(art,i);
+		    }
+		    else
+			s = nullstr;
 		    break;
 #ifdef CONDSUB
 		case '(': {
@@ -567,6 +573,28 @@ char *stoppers;
 		    pattern = cpytill(scrbuf,pattern+1,'?');
 		    if (!*pattern)
 			goto getout;
+		    s = scrbuf;
+		    h = spfbuf;
+		    proc_sprintf = FALSE;
+		    do {
+			switch (*s) {
+			case '^':
+			    *h++ = '\\';
+			    break;
+			case '\\':
+			    *h++ = '\\';
+			    *h++ = '\\';
+			    break;
+			case '%':
+			    proc_sprintf = TRUE;
+			    break;
+			}
+			*h++ = *s;
+		    } while (*s++);
+		    if (proc_sprintf) {
+			dointerp(scrbuf,sizeof scrbuf,spfbuf,Nullch);
+			proc_sprintf = FALSE;
+		    }
 		    if (s = compile(&cond_compex,scrbuf,TRUE,TRUE)) {
 			printf("%s: %s\n",scrbuf,s) FLUSH;
 			pattern += strlen(pattern);
@@ -628,10 +656,13 @@ char *stoppers;
 		    pattern = dointerp(scrbuf,(sizeof scrbuf),pattern+1,"\"");
 		    fputs(scrbuf,stdout) FLUSH;
 		    resetty();
-		    gets(scrbuf);
+		    fgets(scrbuf, sizeof scrbuf, stdin);
 		    noecho();
 		    crmode();
 		    i = strlen(scrbuf);
+		    if (scrbuf[i-1] == '\n') {
+			scrbuf[--i] = '\0';
+		    }
 		    growstr(&input_str, &input_siz, i+1);
 		    safecpy(input_str, scrbuf, i+1);
 		    s = input_str;
@@ -664,20 +695,30 @@ char *stoppers;
 #endif
 		    break;
 		case 'a':
-		    s = scrbuf;
-		    sprintf(s,"%ld",(long)art);
+		    if (in_ng) {
+			s = scrbuf;
+			sprintf(s,"%ld",(long)art);
+		    }
+		    else
+			s = nullstr;
 		    break;
 		case 'A':
+		    if (in_ng) {
 #ifdef LINKART
-		    s = linkartname;	/* so Eunice people get right file */
+			s = linkartname;  /* so Eunice people get right file */
 #else
-		    s = scrbuf;
+			s = scrbuf;
 #ifdef USE_NNTP
-		    sprintf(s,"%s/%s",spool,nntp_artname());
+			if (artopen(art))
+			    nntp_finishbody(FB_SILENT);
+			sprintf(s,"%s/%s",spool,nntp_artname());
 #else
-		    sprintf(s,"%s/%s/%ld",spool,ngdir,(long)art);
+			sprintf(s,"%s/%s/%ld",spool,ngdir,(long)art);
 #endif
 #endif
+		    }
+		    else
+			s = nullstr;
 		    break;
 		case 'b':
 		    s = savedest;
@@ -697,7 +738,10 @@ char *stoppers;
 		    sprintf(s,"%s/%s",spool,ngdir);
 		    break;
 		case 'D':
-		    s = dist_buf = fetchlines(art,DIST_LINE);
+		    if (in_ng)
+			s = dist_buf = fetchlines(art,DIST_LINE);
+		    else
+			s = nullstr;
 		    break;
 		case 'e':
 		    s = (extractprog ? extractprog : "-");
@@ -706,22 +750,30 @@ char *stoppers;
 		    s = extractdest;
 		    break;
 		case 'f':			/* from line */
-		    parseheader(art);
-		    if (htype[REPLY_LINE].ht_minpos >= 0 && !comment_parse) {
+		    if (in_ng) {
+			parseheader(art);
+			if (htype[REPLY_LINE].ht_minpos >= 0 && !comment_parse) {
 						/* was there a reply line? */
-			if (!(s=reply_buf))
-			    s = reply_buf = fetchlines(art,REPLY_LINE);
+			    if (!(s=reply_buf))
+				s = reply_buf = fetchlines(art,REPLY_LINE);
+			}
+			else if (!(s = from_buf))
+			    s = from_buf = fetchlines(art,FROM_LINE);
 		    }
-		    else if (!(s = from_buf))
-			s = from_buf = fetchlines(art,FROM_LINE);
+		    else
+			s = nullstr;
 		    break;
 		case 'F':
-		    parseheader(art);
-		    if (htype[FOLLOW_LINE].ht_minpos >= 0)
+		    if (in_ng) {
+			parseheader(art);
+			if (htype[FOLLOW_LINE].ht_minpos >= 0)
 					/* is there a Followup-To line? */
-			s = follow_buf = fetchlines(art,FOLLOW_LINE);
-		    else 
-			s = ngs_buf = fetchlines(art,NGS_LINE);
+			    s = follow_buf = fetchlines(art,FOLLOW_LINE);
+			else 
+			    s = ngs_buf = fetchlines(art,NGS_LINE);
+		    }
+		    else
+			s = nullstr;
 		    break;
 		case 'h':			/* header file name */
 		    s = headname;
@@ -730,12 +782,16 @@ char *stoppers;
 		    s = phostname;
 		    break;
 		case 'i':
-		    if (!(s=artid_buf))
-			s = artid_buf = fetchlines(art,MESSID_LINE);
-		    if (*s && *s != '<') {
-			sprintf(scrbuf,"<%s>",artid_buf);
-			s = scrbuf;
+		    if (in_ng) {
+			if (!(s=artid_buf))
+			    s = artid_buf = fetchlines(art,MESSID_LINE);
+			if (*s && *s != '<') {
+			    sprintf(scrbuf,"<%s>",artid_buf);
+			    s = scrbuf;
+			}
 		    }
+		    else
+			s = nullstr;
 		    break;
 		case 'I':			/* ref article indicator */
 		    s = scrbuf;
@@ -761,7 +817,10 @@ char *stoppers;
 		    s = scrbuf;
 		    break;
 		case 'n':			/* newsgroups */
-		    s = ngs_buf = fetchlines(art,NGS_LINE);
+		    if (in_ng)
+			s = ngs_buf = fetchlines(art,NGS_LINE);
+		    else
+			s = nullstr;
 		    break;
 		case 'N':			/* full name */
 		    s = getval("NAME",realname);
@@ -806,16 +865,24 @@ char *stoppers;
 		    s = input_str;
 		    break;
 		case 'r':
-		    parseheader(art);
-		    if (htype[REFS_LINE].ht_minpos >= 0) {
-			refs_buf = fetchlines(art,REFS_LINE);
-			refscpy(scrbuf,(sizeof scrbuf),refs_buf);
+		    if (in_ng) {
+			parseheader(art);
+			if (htype[REFS_LINE].ht_minpos >= 0) {
+			    refs_buf = fetchlines(art,REFS_LINE);
+			    refscpy(scrbuf,(sizeof scrbuf),refs_buf);
+			}
+			else
+			    *scrbuf = '\0';
+			s = rindex(scrbuf,'<');
 		    }
 		    else
-			*scrbuf = '\0';
-		    s = rindex(scrbuf,'<');
+			s = nullstr;
 		    break;
 		case 'R':
+		    if (!in_ng) {
+			s = nullstr;
+			break;
+		    }
 		    parseheader(art);
 		    if (htype[REFS_LINE].ht_minpos >= 0) {
 			refs_buf = fetchlines(art,REFS_LINE);
@@ -849,6 +916,10 @@ char *stoppers;
 		    s = scrbuf;
 		    break;
 		case 's':
+		    if (!in_ng) {
+			s = nullstr;
+			break;
+		    }
 		    if (!(s=subj_buf))
 			s = subj_buf = fetchsubj(art,TRUE);
 						/* get subject handy */
@@ -862,6 +933,10 @@ char *stoppers;
 			*h = '\0';
 		    break;
 		case 'S':
+		    if (!in_ng) {
+			s = nullstr;
+			break;
+		    }
 		    if (!(s=subj_buf))
 			s = subj_buf = fetchsubj(art,TRUE);
 						/* get subject handy */
@@ -874,6 +949,10 @@ char *stoppers;
 		    break;
 		case 't':
 		case 'T':
+		    if (!in_ng) {
+			s = nullstr;
+			break;
+		    }
 		    parseheader(art);
 		    if (htype[REPLY_LINE].ht_minpos >= 0) {
 					/* was there a reply line? */
@@ -902,6 +981,10 @@ char *stoppers;
 		case 'U': {
 		    int unseen;
 
+		    if (!in_ng) {
+			s = nullstr;
+			break;
+		    }
 		    unseen = (art <= lastart) && !was_read(art);
 		    if (selected_only) {
 			int selected;
@@ -918,11 +1001,15 @@ char *stoppers;
 		case 'v': {
 		    int selected, unseen;
 
-		    selected = (curr_artp->flags & AF_SEL);
-		    unseen = (art <= lastart) && !was_read(art);
-		    sprintf(scrbuf,"%ld",(long)toread[ng] - selected_count
-					 - (!selected && unseen));
-		    s = scrbuf;
+		    if (in_ng) {
+			selected = (curr_artp->flags & AF_SEL);
+			unseen = (art <= lastart) && !was_read(art);
+			sprintf(scrbuf,"%ld",(long)toread[ng] - selected_count
+						- (!selected && unseen));
+			s = scrbuf;
+		    }
+		    else
+			s = nullstr;
 		    break;
 		}
 		case 'W':
@@ -934,7 +1021,14 @@ char *stoppers;
 		case 'X':			/* rn library */
 		    s = rnlib;
 		    break;
+		case 'y':
+		    s = tmpdir;
+		    break;
 		case 'z':
+		    if (!in_ng) {
+			s = nullstr;
+			break;
+		    }
 #ifdef LINKART
 		    s = linkartname;	/* so Eunice people get right file */
 #else
@@ -1026,11 +1120,18 @@ char *stoppers;
 			abort_interp();
 		}
 		while (*s) {
-		    if (re_quote && index(regexp_specials, *s)) {
+		    if ((re_quote && index(regexp_specials, *s))
+		     || (tick_quote == 2 && *s == '"')) {
 			if (--destsize <= 0)
 			    abort_interp();
 			*dest++ = '\\';
 		    }
+		    else if (re_quote && *s == ' ' && s[1] == ' ') {
+			*dest++ = ' ';
+			*dest++ = '*';
+			while (*++s == ' ') ;
+			continue;
+		    }
 		    else if (tick_quote && *s == '\'') {
 			if ((destsize -= 3) <= 0)
 			    abort_interp();
