@@ -34,6 +34,7 @@
 #include "ngdata.h"
 #include "backpage.h"
 #include "rcln.h"
+#include "sw.h"
 #include "last.h"
 #include "search.h"
 #include "nntp.h"
@@ -105,15 +106,11 @@ char *start_command;			/* command to fake up first */
     char oldmode = mode;
 #ifdef CHARSUBST
     char *whatnext = "%s%sWhat next? [%s]";
+    charsubst = charsets;
 #else
     char *whatnext = "%sWhat next? [%s]";
 #endif
 
-#ifdef ARTSEARCH
-    srchahead = (scanon && !ThreadedGroup	/* did they say -S? */
-	      && ((ART_NUM)toread[ng]) >= scanon ? -1 : 0);
-#endif
-    
     exit_code = NG_NORM;
     localkf_changes = 0;
     killfirst = 0;
@@ -132,6 +129,11 @@ char *start_command;			/* command to fake up first */
     if (!access_ng())
 	return -1;
 
+#ifdef ARTSEARCH
+    srchahead = (scanon && !ThreadedGroup	/* did they say -S? */
+	      && ((ART_NUM)toread[ng]) >= scanon ? -1 : 0);
+#endif
+
     /* FROM HERE ON, RETURN THRU CLEANUP OR WE ARE SCREWED */
 
     forcelast = TRUE;			/* if 0 unread, do not bomb out */
@@ -182,16 +184,14 @@ char *start_command;			/* command to fake up first */
     do_fseek = FALSE;			/* start 1st article at top */
     for (; art<=lastart+1; ) {		/* for each article */
 	mode = 'a';
-#ifdef CHARSUBST
-	charsubst = charsets;
-#endif
 
 	/* do we need to "grow" the newsgroup? */
 
 	if ((art > lastart || forcegrow) && !keep_the_group_static) {
 	    ART_NUM oldlast = lastart;
 #ifdef USE_NNTP
-	    if (!nntp_group(ngname,ng)) {
+	    if ((actfp || getngsize(ng) > lastart)
+	     && !nntp_group(ngname,ng)) {
 		fprintf(stderr,"Your server went south for the winter:\n%s\n",
 			ser_line);
 		finalize(1);
@@ -359,10 +359,11 @@ reask_article:
 	unflush_output();		/* disable any ^O in effect */
 	standout();			/* enter standout mode */
 	/* print prompt, whatever it is */
+	interp(cmd_buf, sizeof cmd_buf, mailcall);
 #ifdef CHARSUBST
-	printf(prompt,mailcall,current_charsubst(),dfltcmd);
+	printf(prompt,cmd_buf,current_charsubst(),dfltcmd);
 #else
-	printf(prompt,mailcall,dfltcmd);
+	printf(prompt,cmd_buf,dfltcmd);
 #endif
 	un_standout();			/* leave standout mode */
 	putchar(' ');
@@ -411,6 +412,7 @@ cleanup:
     kill_unwanted(firstart,"\nCleaning up...\n\n",FALSE);
 					/* do cleanup from KILL file, if any */
 #endif
+    chase_xrefs(FALSE);
     in_ng = FALSE;			/* leave newsgroup state */
     if (artfp != Nullfp) {		/* article still open? */
 	fclose(artfp);			/* close it */
@@ -505,7 +507,7 @@ s to mark subthread unread.\n";
 	    IF(verbose)
 	    {
 		fputs("\
-Type + to enter select thread mode using all the unread articles.\n\
+Type + to enter select thread mode using all the already-read articles.\n\
 (The selected threads will be marked as unread and displayed as usual.)\n\
 ",stdout) FLUSH;
 		fputs(u_help_thread,stdout);
@@ -700,7 +702,7 @@ This is the last leaf in this tree.\n",stdout) FLUSH;
 	return AS_ASK;
     case ':':			/* execute command on selected articles */
 	page_line = 1;
-	if (!use_selected())
+	if (!thread_perform())
 	    return AS_INP;
 	putchar('\n');
 	art = curr_art;
@@ -992,7 +994,7 @@ run_the_selector:
     case '=': {			/* list subjects */
 	char tmpbuf[256];
 	ART_NUM oldart = art;
-	int cmd;
+	int cmd, len;
 	char *s;
 	char *subjline = getval("SUBJLINE",Nullch);
 	ARTICLE *ap = article_ptr(firstart);
@@ -1001,12 +1003,13 @@ run_the_selector:
 	for (i=firstart; i<=lastart && !int_count; i++, ap++) {
 	    if (!(ap->flags & AF_READ) && (s = fetchsubj(i,FALSE)) != Nullch) {
 		sprintf(tmpbuf,"%5ld ", i);
+		len = strlen(tmpbuf);
 		if (subjline) {
 		    art = i;
-		    interp(tmpbuf + 6, (sizeof tmpbuf) - 6, subjline);
+		    interp(tmpbuf + len, sizeof tmpbuf - len, subjline);
 		}
 		else
-		    safecpy(tmpbuf + 6, s, (sizeof tmpbuf) - 6);
+		    safecpy(tmpbuf + len, s, sizeof tmpbuf - len);
 		if (cmd = print_lines(tmpbuf,NOMARKING)) {
 		    if (cmd > 0)
 			pushchar(cmd);
@@ -1085,6 +1088,19 @@ run_the_selector:
 	setmail(TRUE);		/* force a mail check */
 #endif
 	return AS_ASK;
+#ifdef INNERSEARCH
+    case Ctl('e'):
+	if (art <= lastart) {
+	    reread = TRUE;
+	    do_fseek = TRUE;
+	    innerlight = artline - 1;
+	    topline = artline;
+	    innersearch = artsize;
+	    gline = 0;
+	    hide_everything = 'b';
+	}
+	return AS_NORM;
+#endif
     case 'B':				/* back up one line */
     case 'b': case Ctl('b'):		/* back up a page */
 	if (art <= lastart) {
@@ -1098,7 +1114,6 @@ run_the_selector:
 	    else {
 		target = topline - (LINES - 2);
 		if (marking && (marking_areas & BACKPAGE_MARKING)) {
-		    extern ART_LINE highlight;
 		    highlight = topline;
 		}
 	    }
@@ -1277,13 +1292,14 @@ ask_catchup()
 {
     char ch;
     bool use_one_line = (mode == 't');
+    int leave = 0;
 
     if (!use_one_line)
 	putchar('\n') FLUSH;
 reask_catchup:
 #ifdef VERBOSE
     IF(verbose)
-	in_char("Do you really want to mark everything as read?",'C',"yn");
+	in_char("Do you really want to mark everything as read?",'C',"yn#h");
     ELSE
 #endif
 #ifdef TERSE
@@ -1299,6 +1315,7 @@ reask_catchup:
 	    fputs("\n\
 Type y or SP to mark all articles as read.\n\
 Type n to leave articles marked as they are.\n\
+Enter a number to mark all but the last # articles as read.\n\
 Type u to mark everything read and unsubscribe.\n\n\
 ",stdout) FLUSH;
 	ELSE
@@ -1318,6 +1335,24 @@ u to mark all and unsubscribe.\n\n\
 	putchar('\n') FLUSH;
 	return 'n';
     }
+    if (ch == '#') {
+	use_one_line = FALSE;
+	in_char("\nEnter the number of articles to leave unread: ", 'C', "0");
+	if ((ch = *buf) == '0')
+	    ch = 'y';
+    }
+    if (isdigit(ch)) {
+	buf[1] = FINISHCMD;
+	if (!finish_command(FALSE)) {
+	    use_one_line = FALSE;
+	    putchar('\n') FLUSH;
+	    goto reask_catchup;
+	}
+	else {
+	    leave = atoi(buf);
+	    ch = 'y';
+	}
+    }
     if (ch != 'y' && ch != 'u') {
 	use_one_line = FALSE;
 	printf("\n%s\n", hforhelp) FLUSH;
@@ -1326,12 +1361,12 @@ u to mark all and unsubscribe.\n\n\
     }
     if (mode == 'n') {
 	putchar('\n') FLUSH;
-	catch_up(ng);
+	catch_up(ng, leave);
     }
     else {
 	int i;
 	ARTICLE *ap;
-	for (i = firstart, ap = article_ptr(i); i <= lastart; i++, ap++)
+	for (i = firstart, ap = article_ptr(i); i <= lastart-leave; i++, ap++)
 	    ap->flags = ((ap->flags & ~sel_mask) | AF_READ);
 	selected_count = selected_subj_cnt = selected_only = 0;
 	toread[ng] = 0;
@@ -1409,7 +1444,7 @@ q aborts.\n\n\
 	} else
 	    ch = (use_one_line? '+' : '.');
 	if (thread_cmd)
-	    select_arts_thread(artp, AF_AUTOSELECTALL);
+	    select_arts_thread(artp, AUTO_SELECTALL);
 	else
 	    select_arts_subject(artp, 0);
 	if (mode != 't')
@@ -1421,7 +1456,7 @@ q aborts.\n\n\
 	    artp = artp_hold;
 	} else
 	    ch = (use_one_line? '+' : '.');
-	select_subthread(artp,thread_cmd? AF_AUTOSELECT : 0);
+	select_subthread(artp,thread_cmd? AUTO_SELECT : 0);
 	if (mode != 't')
 	    printf("\nSelection memorized.\n");
     } else if (ch == 'j') {
