@@ -17,6 +17,7 @@
 #include "common.h"
 #include "term.h"
 #include "util.h"
+#include "util2.h"
 #include "cache.h"
 #include "bits.h"
 #include "ngdata.h"
@@ -223,20 +224,35 @@ numnum()
 }
 
 int
-use_selected()
+thread_perform()
 {
     register SUBJECT *sp;
     register ARTICLE *ap;
     bool want_read;
     char *cmdstr;
     int len, ret = 1;
+    int bits;
+    bool one_thread = FALSE;
 
     if (!finish_command(TRUE))	/* get rest of command */
 	return 0;
     if (!buf[1])
 	return -1;
-    cmdstr = savestr(buf+1);
-    want_read = (sel_rereading || *cmdstr =='m');
+    len = 1;
+    if (buf[1] == ':') {
+	bits = 0;
+	len++;
+    }
+    else
+	bits = SF_VISIT;
+    if (buf[len] == '.') {
+	if (!artp)
+	    return -1;
+	one_thread = TRUE;
+	len++;
+    }
+    cmdstr = savestr(buf+len);
+    want_read = (sel_rereading || *cmdstr == 'm');
 
     perform_cnt = 0;
     page_line = 1;
@@ -247,7 +263,13 @@ use_selected()
      || (len == 2
       && (((*cmdstr == '+' || *cmdstr == '-') && cmdstr[0] == cmdstr[1])
        || *cmdstr == 'T'))) {
-	for (sp = next_subj(Nullsubj,SF_VISIT); sp; sp = next_subj(sp,SF_VISIT)) {
+	if (one_thread)
+	    sp = (sel_mode==SM_THREAD? artp->subj->thread->subj : artp->subj);
+	else
+	    sp = next_subj(Nullsubj,bits);
+	for ( ; sp; sp = next_subj(sp,bits)) {
+	    if (!(sp->flags & sel_mask) ^ !bits)
+		continue;
 	    artp = first_art(sp);
 	    if (artp) {
 		art = article_num(artp);
@@ -261,6 +283,8 @@ use_selected()
 		if (mode != 't' && *cmdstr != 't' && *cmdstr != 'T')
 		    putchar('\n') FLUSH;
 #endif
+	    if (one_thread)
+		break;
 	}
     } else if (strEQ(cmdstr, "E")) {
 	/* The 'E'nd-decode command doesn't do any looping at all. */
@@ -268,14 +292,24 @@ use_selected()
 	    decode_end();
 	else
 	    ret = 2;
+    } else if (*cmdstr == 'p') {
+	ART_NUM oldart = art;
+	art = lastart+1;
+	followup();
+	forcegrow = TRUE;
+	art = oldart;
     } else {
-	/* The rest loop through all (selected) articles. */
+	/* The rest loop through the articles. */
 	/* Use the explicit article-order if it exists */
 	if (artptr_list) {
 	    ARTICLE **app, **limit = artptr_list + artptr_list_size;
-	    for (app = artptr_list; app < limit; app++)
-		if ((!((ap = *app)->flags & AF_READ) ^ want_read)
-		 && (ap->flags & sel_mask)) {
+	    sp = (sel_mode==SM_THREAD? artp->subj->thread->subj : artp->subj);
+	    for (app = artptr_list; app < limit; app++) {
+		ap = *app;
+		if (one_thread && ap->subj->thread != sp->thread)
+		    continue;
+		if ((!(ap->flags & AF_READ) ^ want_read)
+		 && !(ap->flags & sel_mask) ^ !!bits) {
 		    art = article_num(ap);
 		    artp = ap;
 		    if (perform(cmdstr, TRUE)) {
@@ -283,11 +317,16 @@ use_selected()
 			goto break_out;
 		    }
 		}
+	    }
 	} else {
-	    for (sp = next_subj(Nullsubj,SF_VISIT); sp; sp = next_subj(sp,SF_VISIT))
+	    if (one_thread)
+		sp = (sel_mode==SM_THREAD? artp->subj->thread->subj : artp->subj);
+	    else
+		sp = next_subj(Nullsubj,bits);
+	    for ( ; sp; sp = next_subj(sp,bits)) {
 		for (ap = first_art(sp); ap; ap = next_art(ap))
 		    if ((!(ap->flags & AF_READ) ^ want_read)
-		     && (ap->flags & sel_mask)) {
+		     && !(ap->flags & sel_mask) ^ !!bits) {
 			art = article_num(ap);
 			artp = ap;
 			if (perform(cmdstr, TRUE)) {
@@ -295,6 +334,9 @@ use_selected()
 			    goto break_out;
 			}
 		    }
+		if (one_thread)
+		    break;
+	    }
 	}
     }
   break_out:
@@ -331,15 +373,15 @@ int toplevel;
 	} else if (ch == '+') {
 	    if (saveit || cmdlst[1] == '+') {
 		if (sel_mode == SM_THREAD)
-		    select_arts_thread(artp, saveit? AF_AUTOSELECTALL : 0);
+		    select_arts_thread(artp, saveit? AUTO_SELECTALL : 0);
 		else
-		    select_arts_subject(artp, saveit? AF_AUTOSELECTALL : 0);
+		    select_arts_subject(artp, saveit? AUTO_SELECTALL : 0);
 		if (cmdlst[1] == '+')
 		    cmdlst++;
 	    } else
-		select_article(artp, AF_ECHO);
+		select_article(artp, AUTO_ECHO);
 	} else if (ch == '.') {
-	    select_subthread(artp, saveit? AF_AUTOSELECT : 0);
+	    select_subthread(artp, saveit? AUTO_SELECT : 0);
 	} else if (ch == '-') {
 	    if (cmdlst[1] == '-') {
 		if (sel_mode == SM_THREAD)
