@@ -7,11 +7,21 @@
 
 #ifdef USE_NNTP
 
+#define FB_BACKGROUND	0
+#define FB_OUTPUT	1
+#define FB_SILENT	2
+
+int	nntp_list _((char*,char*,int));
 bool	nntp_group _((char*,NG_NUM));
 bool	nntp_stat _((ART_NUM));
 ART_NUM	nntp_stat_id _((char*));
 bool	nntp_header _((ART_NUM));
 FILE	*nntp_body _((ART_NUM));
+long	nntp_artsize _((void));
+int	nntp_finishbody _((int));
+void	nntp_seekart _((long));
+long	nntp_tellart _((void));
+char	*nntp_readart _((char*,int));
 time_t	nntp_time _((void));
 bool	nntp_newgroups _((time_t));
 bool	nntp_listgroup _((void));
