@@ -17,7 +17,7 @@ EXT ART_NUM *ngmax INIT(NULL);
 
 void    rcln_init _((void));
 #ifdef CATCHUP
-void	catch_up _((NG_NUM));
+void	catch_up _((NG_NUM,int));
 #endif
 int	addartnum _((ART_NUM,char*));
 #ifdef MCHASE
@@ -25,4 +25,4 @@ void	subartnum _((ART_NUM,char*));
 #endif
 void	prange _((char*,ART_NUM,ART_NUM));
 void	set_toread _((NG_NUM));
-void	checkexpired _((NG_NUM));
+void	checkexpired _((NG_NUM,ART_NUM));
