@@ -297,6 +297,7 @@ int flags;
 	dingaling();
 	printf("\nBad newsgroup name.\n") FLUSH;
       check_fuzzy_match:
+#ifdef EDIT_DISTANCE
 	if (fuzzyGet && (flags & GNG_FUZZY)) {
 	    flags &= ~GNG_FUZZY;
 	    if (find_close_match())
@@ -304,6 +305,7 @@ int flags;
 	    else
 		return FALSE;
 	} else
+#endif
 	    return FALSE;
     }
     set_ngname(what);
@@ -1002,10 +1004,10 @@ write_rc()
 	}
     }
     fflush(rcfp);
-    if (ferror(rcfp))
+    /* fclose is the only sure test for full disks via NFS */
+    if (ferror(rcfp) || fclose(rcfp) == EOF)
 	goto write_error;
 
-    fclose(rcfp);			/* close .newnewsrc */
     UNLINK(rcname);
 #ifdef HAS_RENAME
     rename(rctname,rcname);
