@@ -77,6 +77,9 @@ char_int ch;
     case 'c':
 	sel_sort = SS_COUNT;
 	break;
+    case 'n':
+	sel_sort = SS_NUMBER;
+	break;
     case 'g':
 	sel_sort = SS_GROUPS;
 	break;
@@ -104,7 +107,8 @@ int ssort;
     if (sel_mode == SM_ARTICLE) {
 	if (sel_sort == SS_COUNT)
 	    sel_sort = SS_DATE;
-    } else if (sel_sort == SS_AUTHOR || sel_sort == SS_GROUPS)
+    } else if (sel_sort == SS_AUTHOR || sel_sort == SS_GROUPS
+	    || sel_sort == SS_NUMBER)
 	sel_sort = SS_DATE;
 
     switch (sel_mode) {
@@ -137,6 +141,9 @@ int ssort;
     case SS_COUNT:
 	sel_sort_string = "count";
 	break;
+    case SS_NUMBER:
+	sel_sort_string = "number";
+	break;
     case SS_GROUPS:
 	sel_sort_string = "SubjDate";
 	break;
