@@ -45,7 +45,7 @@ ART_NUM artnum;
     } else {
 	article = (ARTICLE *)safemalloc(sizeof (ARTICLE));
 	bzero((char*)article, sizeof (ARTICLE));
-	article->flags |= AF_READ|AF_FAKE|AF_TMPMEM;
+	article->flags |= AF_READ|AF_FAKE|AF_MISSING|AF_TMPMEM;
     }
     return article;
 }
@@ -110,8 +110,8 @@ ARTICLE *article;
 	    article->child1 = fake_ap->child1;
 	    article->sibling = fake_ap->sibling;
 	    fake_had_subj = fake_ap->subj;
-	    if (fake_ap->flags & AF_AUTOFLAGS) {
-		article->flags |= fake_ap->flags & AF_AUTOFLAGS;
+	    if (fake_ap->autofl) {
+		article->autofl |= fake_ap->autofl;
 		localkf_changes = 2;
 	    }
 	    if (curr_artp == fake_ap) {
@@ -197,10 +197,10 @@ ARTICLE *article;
 {
     register ARTICLE *ap, *last;
     register char *cp, *end;
-    ARTICLE *kill_ap = ((article->flags & AF_AUTOKILL)? article : Nullart);
-    int art_chain_flags = article->flags
-	| (article->subj->articles? article->subj->articles->flags : 0);
-    int art_thread_flags;
+    ARTICLE *kill_ap = ((article->autofl & AUTO_KILL)? article : Nullart);
+    int art_chain_autofl = article->autofl
+	| (article->subj->articles? article->subj->articles->autofl : 0);
+    int art_thread_autofl;
 
     /* We're definitely not a fake anymore */
     article->flags = (article->flags & ~AF_FAKE) | AF_THREADED;
@@ -220,25 +220,6 @@ ARTICLE *article;
 	}
 	stopper = ap;
 	unlink_child(article);
-#if 0
-	if (ap) {			/* do we have decent parents? */
-	    /* Yes: assume that our references are ok, and just reorder us
-	    ** with our siblings by date.
-	    */
-	    link_child(article);
-	    /* Freshen the date & subject in any faked parent articles. */
-	    for (ap = article->parent;
-		 ap && (ap->flags&AF_FAKE)==AF_FAKE && article->date < ap->date;
-		 ap = ap->parent)
-	    {
-		ap->date = article->date;
-		ap->subj = article->subj;
-		unlink_child(ap);
-		link_child(ap);
-	    }
-	    goto exit;
-	}
-#endif
 	/* We'll assume that this article has as good or better references
 	** than the child that faked us initially.  Free the fake reference-
 	** chain and process our references as usual.
@@ -275,8 +256,8 @@ ARTICLE *article;
 		break;
 	    ap = get_article(cp);
 	    *cp = '\0';
-	    art_chain_flags |= ap->flags;
-	    if (ap->flags & AF_AUTOKILL)
+	    art_chain_autofl |= ap->autofl;
+	    if (ap->autofl & AUTO_KILL)
 		kill_ap = ap;
 
 	    /* Check for duplicates on the reference line.  Brand-new data has
@@ -343,28 +324,25 @@ ARTICLE *article;
 	*/
 	link_child(article);
     }
-#if 0
-exit:
-#endif
     if (!(article->flags & AF_CACHED))
 	cache_article(article);
-    art_thread_flags = art_chain_flags;
+    art_thread_autofl = art_chain_autofl;
     if (sel_mode == SM_THREAD && article == article->subj->articles) {
 	SUBJECT *sp = article->subj->thread_link;
 	while (sp != article->subj) {
 	    if (sp->articles)
-		art_thread_flags |= sp->articles->flags;
+		art_thread_autofl |= sp->articles->autofl;
 	    sp = sp->thread_link;
 	}
     }
-    if (art_thread_flags & AF_AUTOSELECTALL) {
+    if (art_thread_autofl & AUTO_SELECTALL) {
 	if (sel_mode == SM_THREAD)
-	    select_arts_thread(article, AF_AUTOSELECTALL);
+	    select_arts_thread(article, AUTO_SELECTALL);
 	else
-	    select_arts_subject(article, AF_AUTOSELECTALL);
-    } else if (art_chain_flags & AF_AUTOSELECT)
-	select_subthread(article, AF_AUTOSELECT);
-    if (art_thread_flags & AF_AUTOKILLALL) {
+	    select_arts_subject(article, AUTO_SELECTALL);
+    } else if (art_chain_autofl & AUTO_SELECT)
+	select_subthread(article, AUTO_SELECT);
+    if (art_thread_autofl & AUTO_KILLALL) {
 	if (sel_mode == SM_THREAD)
 	    kill_arts_thread(article, KF_ALL|KF_KILLFILE);
 	else
