@@ -241,7 +241,8 @@ reask_selector:
 		(!sel_prior_arts? "Top " : nullstr),
 		(long)((sel_prior_arts+sel_page_arts)*100 / sel_total_arts),
 		page_char, end_char);
-    sprintf(promptbuf, "%s-- %s %s (%s%s order) -- %s", mailcall,
+    interp(buf, sizeof buf, mailcall);
+    sprintf(promptbuf, "%s-- %s %s (%s%s order) -- %s", buf,
 	    sel_exclusive? "SELECTED" : "Select", sel_mode_string,
 	    sel_direction<0? "reverse " : nullstr, sel_sort_string, cmd_buf);
 #ifdef CLEAREOL
@@ -740,8 +741,8 @@ q does nothing.\n\n\
 	erase_eol();		/* erase the prompt */
     reask_sort:
 	if (sel_mode == SM_ARTICLE)
-	    in_char("Order by Date, Subject, Author, subject-date Groups?",
-		    'q', "dsagDSAG");
+	    in_char("Order by Date, Subject, Author, Number, subject-date Groups?",
+		    'q', "dsangDSANG");
 	else
 	    in_char("Order by Date, Subject, or Count?", 'q', "dscDSC");
 #ifdef VERIFY
@@ -757,6 +758,7 @@ Type s to order the items by subject.\n\
 		if (sel_mode == SM_ARTICLE)
 		    fputs("\
 Type a to order the items by author.\n\
+Type n to order the items by number.\n\
 Type g to order the items in subject-groups by date.\n\
 ",stdout) FLUSH;
 		else
@@ -824,6 +826,7 @@ q does nothing.\n\n\
 	sel_page_app = Null(ARTICLE**);
 	init_pages(FILL_LAST_PAGE);
 	empty_ok = TRUE;
+	sel_item_index = 0;
 	return DS_DISPLAY;
     case 'X':  case 'D':  case 'J':
 	if (!sel_rereading) {
@@ -933,7 +936,20 @@ q does nothing.\n\n\
 	edit_kfile();
 	xmouse_on();
 	return DS_DISPLAY;
-    case ':':  case '/':  case '&':  case '!':
+    case ':':
+	if (sel_mode == SM_ARTICLE)
+	    artp = (ARTICLE*)sel_items[sel_item_index].ptr;
+	else {
+	    register SUBJECT *sp = (SUBJECT*)sel_items[sel_item_index].ptr;
+	    if (sel_mode == SM_THREAD) {
+		while (!sp->misc)
+		    sp = sp->thread_link;
+	    }
+	    artp = sp->articles;
+	}
+	art = article_num(artp);
+	/* FALL THROUGH */
+    case '/':  case '&':  case '!':
 	erase_eol();		/* erase the prompt */
 	if (!finish_command(TRUE)) {	/* get rest of command */
 	    if (clean_screen)
@@ -952,7 +968,7 @@ q does nothing.\n\n\
 	    int sel_art_save = selected_count;
 
 	    if (ch == ':') {
-		clean_screen = (use_selected() == 2) && clean_screen;
+		clean_screen = (thread_perform() == 2) && clean_screen;
 		if (!sel_rereading) {
 		    register SUBJECT *sp;
 		    for (sp = first_subject; sp; sp = sp->next) {
@@ -1022,7 +1038,7 @@ q does nothing.\n\n\
 	    if (ch > 0) {
 		/* try to optimize the screen update for some commands. */
 		if (!index(sel_chars, ch)
-		 && (index("<+>^$!?&:/hDEJLNOPqQRSUXYZ\n\r\t\033", ch)
+		 && (index("<+>^$!?&:;/hDEJLNOPqQRSUXYZ\n\r\t\033", ch)
 		  || ch == Ctl('k'))) {
 		    buf[0] = sel_ret = ch;
 		    buf[1] = FINISHCMD;
