@@ -38,10 +38,10 @@ HASHTABLE *msgid_hash = 0;
 void
 thread_init()
 {
-    if (try_ov)
-	try_ov = ov_init();
-    if (try_mt)
-	try_mt = mt_init();
+    if (try_ov > 0)
+	try_ov = ov_init()? 1 : -1;
+    if (try_mt > 0)
+	try_mt = mt_init()? 1 : -1;
 }
 
 /* Generate the thread data we need for this group.  We must call
@@ -67,10 +67,10 @@ thread_open()
     else
 	set_selector(sel_threadmode, sel_threadsort);
 
-    if (try_mt && !first_subject)
+    if (try_mt > 0 && !first_subject)
 	if (!mt_data())
 	    return;
-    if (try_ov && first_cached > last_cached)
+    if (try_ov > 0 && first_cached > last_cached)
 	if (thread_always)
 	    (void) ov_data(absfirst, lastart, FALSE);
 	else if (firstart > lastart) {
@@ -151,6 +151,8 @@ top_article()
     art = lastart+1;
     artp = Nullart;
     inc_art(selected_only, FALSE);
+    if (art > lastart && last_cached < lastart)
+	art = firstart;
 }
 
 ARTICLE *
@@ -461,11 +463,9 @@ bool
 next_art_with_subj()
 {
     register ARTICLE *ap = artp;
-    register SUBJECT *sp;
 
     if (!ap)
 	return FALSE;
-    sp = ap->subj;
 
     do {
 	ap = ap->subj_next;
@@ -491,11 +491,9 @@ bool
 prev_art_with_subj()
 {
     register ARTICLE *ap = artp, *ap2;
-    register SUBJECT *sp;
 
     if (!ap)
 	return FALSE;
-    sp = ap->subj;
 
     do {
 	ap2 = ap->subj->articles;
@@ -565,23 +563,23 @@ int subj_mask;
 /* Select a single article.
 */
 void
-select_article(ap, sel_flags)
+select_article(ap, auto_flags)
 register ARTICLE *ap;
-int sel_flags;
+int auto_flags;
 {
     int desired_flags = (sel_rereading? AF_READ : 0);
 #ifdef VERBOSE
     bool echo;
 
-    if (sel_flags & AF_ECHO) {
+    if (auto_flags & AUTO_ECHO) {
 	echo = TRUE;
-	sel_flags &= ~AF_ECHO;
+	auto_flags &= ~AUTO_ECHO;
     } else
 	echo = FALSE;
 #else
-    sel_flags &= ~AF_ECHO;
+    auto_flags &= ~AUTO_ECHO;
 #endif
-    if (sel_flags & (AF_AUTOSELECT|AF_AUTOSELECTALL))
+    if (auto_flags & (AUTO_SELECT|AUTO_SELECTALL))
 	localkf_changes = 2;
     if ((ap->flags & (AF_MISSING|AF_READ)) == desired_flags) {
 	if (!(ap->flags & sel_mask)) {
@@ -593,9 +591,10 @@ int sel_flags;
 	    }
 #endif
 	}
-	ap->flags = (ap->flags & ~AF_DEL) | sel_mask | sel_flags;
+	ap->flags = (ap->flags & ~AF_DEL) | sel_mask;
+	ap->autofl |= auto_flags;
     } else
-	ap->flags |= sel_flags;
+	ap->autofl |= auto_flags;
     if (ap->subj) {
 	if (!(ap->subj->flags & sel_mask))
 	    selected_subj_cnt++;
@@ -607,35 +606,36 @@ int sel_flags;
 /* Select this article's subject.
 */
 void
-select_arts_subject(ap, sel_flags)
+select_arts_subject(ap, auto_flags)
 register ARTICLE *ap;
-int sel_flags;
+int auto_flags;
 {
     if (ap->subj && ap->subj->articles)
-	select_subject(ap->subj, sel_flags);
+	select_subject(ap->subj, auto_flags);
     else
-	select_article(ap, sel_flags);
+	select_article(ap, auto_flags);
 }
 
 /* Select all the articles in a subject.
 */
 void
-select_subject(subj, sel_flags)
+select_subject(subj, auto_flags)
 SUBJECT *subj;
-int sel_flags;
+int auto_flags;
 {
     register ARTICLE *ap;
     int desired_flags = (sel_rereading? AF_READ : 0);
     int old_count = selected_count;
 
-    if (sel_flags & (AF_AUTOSELECT|AF_AUTOSELECTALL))
+    if (auto_flags & (AUTO_SELECT|AUTO_SELECTALL))
 	localkf_changes = 2;
     for (ap = subj->articles; ap; ap = ap->subj_next) {
 	if ((ap->flags & (AF_MISSING|AF_READ|sel_mask)) == desired_flags) {
-	    ap->flags |= sel_mask | sel_flags;
+	    ap->flags |= sel_mask;
+	    ap->autofl |= auto_flags;
 	    selected_count++;
 	} else
-	    ap->flags |= sel_flags;
+	    ap->autofl |= auto_flags;
     }
     if (selected_count > old_count) {
 	if (!(subj->flags & sel_mask))
@@ -650,28 +650,28 @@ int sel_flags;
 /* Select this article's thread.
 */
 void
-select_arts_thread(ap, sel_flags)
+select_arts_thread(ap, auto_flags)
 register ARTICLE *ap;
-int sel_flags;
+int auto_flags;
 {
     if (ap->subj && ap->subj->thread)
-	select_thread(ap->subj->thread, sel_flags);
+	select_thread(ap->subj->thread, auto_flags);
     else
-	select_arts_subject(ap, sel_flags);
+	select_arts_subject(ap, auto_flags);
 }
 
 /* Select all the articles in a thread.
 */
 void
-select_thread(thread, sel_flags)
+select_thread(thread, auto_flags)
 register ARTICLE *thread;
-int sel_flags;
+int auto_flags;
 {
     register SUBJECT *sp;
 
     sp = thread->subj;
     do {
-	select_subject(sp, sel_flags);
+	select_subject(sp, auto_flags);
 	sp = sp->thread_link;
     } while (sp != thread->subj);
 }
@@ -679,9 +679,9 @@ int sel_flags;
 /* Select the subthread attached to this article.
 */
 void
-select_subthread(ap, sel_flags)
+select_subthread(ap, auto_flags)
 register ARTICLE *ap;
-int sel_flags;
+int auto_flags;
 {
     register ARTICLE *limit;
     SUBJECT *subj;
@@ -698,14 +698,15 @@ int sel_flags;
 	}
     }
 
-    if (sel_flags & (AF_AUTOSELECT|AF_AUTOSELECTALL))
+    if (auto_flags & (AUTO_SELECT|AUTO_SELECTALL))
 	localkf_changes = 2;
     for (; ap != limit; ap = bump_art(ap)) {
 	if ((ap->flags & (AF_MISSING|AF_READ|sel_mask)) == desired_flags) {
-	    ap->flags |= sel_mask | sel_flags;
+	    ap->flags |= sel_mask;
+	    ap->autofl |= auto_flags;
 	    selected_count++;
 	} else
-	    ap->flags |= sel_flags;
+	    ap->autofl |= auto_flags;
     }
     if (subj && selected_count > old_count) {
 	if (!(subj->flags & sel_mask))
@@ -821,37 +822,37 @@ deselect_all()
 /* Kill all unread articles attached to this article's subject.
 */
 void
-kill_arts_subject(ap, kill_flags)
+kill_arts_subject(ap, auto_flags)
 register ARTICLE *ap;
-int kill_flags;
+int auto_flags;
 {
     if (ap->subj && ap->subj->articles)
-	kill_subject(ap->subj, kill_flags);
+	kill_subject(ap->subj, auto_flags);
     else {
 	set_read(ap);
-	ap->flags |= AF_AUTOKILLALL;
+	ap->autofl |= AUTO_KILLALL;
     }
 }
 
 /* Kill all unread articles attached to the given subject.
 */
 void
-kill_subject(subj, kill_flags)
+kill_subject(subj, auto_flags)
 SUBJECT *subj;
-int kill_flags;
+int auto_flags;
 {
     register ARTICLE *ap;
-    register int killmask = ((kill_flags&KF_ALL)? AF_READ:(AF_READ|sel_mask));
+    register int killmask = ((auto_flags&KF_ALL)? AF_READ:(AF_READ|sel_mask));
 
-    if (kill_flags & KF_KILLFILE) {
+    if (auto_flags & KF_KILLFILE) {
 	localkf_changes = 2;
-	kill_flags = AF_AUTOKILLALL;
+	auto_flags = AUTO_KILLALL;
     } else
-	kill_flags = 0;
+	auto_flags = 0;
     for (ap = subj->articles; ap; ap = ap->subj_next) {
 	if (!(ap->flags & killmask))
 	    set_read(ap);
-	ap->flags |= kill_flags;
+	ap->autofl |= auto_flags;
     }
     subj->flags &= ~(SF_VISIT | SF_WASSELECTED);
 }
@@ -859,28 +860,28 @@ int kill_flags;
 /* Kill all unread articles attached to this article's thread.
 */
 void
-kill_arts_thread(ap, kill_flags)
+kill_arts_thread(ap, auto_flags)
 register ARTICLE *ap;
-int kill_flags;
+int auto_flags;
 {
     if (ap->subj && ap->subj->thread)
-	kill_thread(ap->subj->thread, kill_flags);
+	kill_thread(ap->subj->thread, auto_flags);
     else
-	kill_arts_subject(ap, kill_flags);
+	kill_arts_subject(ap, auto_flags);
 }
 
 /* Kill all unread articles attached to the given thread.
 */
 void
-kill_thread(thread, kill_flags)
+kill_thread(thread, auto_flags)
 register ARTICLE *thread;
-int kill_flags;
+int auto_flags;
 {
     register SUBJECT *sp;
 
     sp = thread->subj;
     do {
-	kill_subject(sp, kill_flags);
+	kill_subject(sp, auto_flags);
 	sp = sp->thread_link;
     } while (sp != thread->subj);
 }
@@ -888,9 +889,9 @@ int kill_flags;
 /* Kill the subthread attached to this article.
 */
 void
-kill_subthread(ap, kill_flags)
+kill_subthread(ap, auto_flags)
 register ARTICLE *ap;
-int kill_flags;
+int auto_flags;
 {
     register ARTICLE *limit;
 
@@ -903,15 +904,15 @@ int kill_flags;
 	}
     }
 
-    if (kill_flags & KF_KILLFILE) {
+    if (auto_flags & KF_KILLFILE) {
 	localkf_changes = 2;
-	kill_flags = AF_AUTOKILL;
+	auto_flags = AUTO_KILL;
     } else
-	kill_flags = 0;
+	auto_flags = 0;
     for (; ap != limit; ap = bump_art(ap)) {
 	if (!(ap->flags & (AF_READ|AF_MISSING)))
 	    set_read(ap);
-	ap->flags |= kill_flags;
+	ap->autofl |= auto_flags;
     }
 }
 
@@ -1004,7 +1005,7 @@ SUBJECT *subj;
     register ARTICLE *ap;
 
     for (ap = subj->articles; ap; ap = ap->subj_next) {
-	ap->flags &= ~AF_AUTOFLAGS;
+	ap->autofl = 0;
     }
     localkf_changes = 2;
 }
@@ -1042,7 +1043,7 @@ register ARTICLE *ap;
     }
 
     for (; ap != limit; ap = bump_art(ap)) {
-	ap->flags &= ~AF_AUTOFLAGS;
+	ap->autofl = 0;
     }
     localkf_changes = 2;
 }
@@ -1453,8 +1454,7 @@ sort_subjects()
 
     switch (sel_sort) {
     case SS_DATE:
-    case SS_AUTHOR:
-    case SS_GROUPS:
+    default:
 	sort_procedure = (sel_mode == SM_THREAD?
 			  threadorder_date : subjorder_date);
 	break;
@@ -1466,8 +1466,6 @@ sort_subjects()
 	sort_procedure = (sel_mode == SM_THREAD?
 			  threadorder_count : subjorder_count);
 	break;
-    default:
-	return;
     }
 
     subj_list = (SUBJECT**)safemalloc(subject_count * sizeof (SUBJECT*));
@@ -1528,6 +1526,14 @@ register ARTICLE **art2;
     return (int)((*art1)->date - (*art2)->date);
 }
 
+int
+artorder_number(art1, art2)
+register ARTICLE **art1;
+register ARTICLE **art2;
+{
+    return (int)(article_num(*art1) - article_num(*art2)) * sel_direction;
+}
+
 int
 artorder_groups(art1, art2)
 register ARTICLE **art1;
@@ -1553,7 +1559,7 @@ sort_articles()
 
     switch (sel_sort) {
     case SS_DATE:
-    case SS_COUNT:
+    default:
 	sort_procedure = artorder_date;
 	break;
     case SS_SUBJECT:
@@ -1562,11 +1568,12 @@ sort_articles()
     case SS_AUTHOR:
 	sort_procedure = artorder_author;
 	break;
+    case SS_NUMBER:
+	sort_procedure = artorder_number;
+	break;
     case SS_GROUPS:
 	sort_procedure = artorder_groups;
 	break;
-    default:
-	return;
     }
     sel_page_app = 0;
     qsort(artptr_list, artptr_list_size, sizeof (ARTICLE*), sort_procedure);
