@@ -16,11 +16,14 @@
 #include "EXTERN.h"
 #include "common.h"
 #include "util.h"
+#include "util2.h"
 #include "cache.h"
 #include "head.h"
 #include "only.h"
 #include "term.h"
+#include "trn.h"
 #include "ng.h"
+#include "ngdata.h"
 #include "artstate.h"
 #include "intrp.h"
 #include "rt-page.h"
@@ -161,6 +164,7 @@ register char *s;
 #endif
     if (*s != '-' && *s != '+') {	/* newsgroup pattern */
 	setngtodo(s);
+	emptyOnly = TRUE;
     }
     else {				/* normal switch */
 	bool upordown = *s == '-' ? TRUE : FALSE;
@@ -283,7 +287,7 @@ register char *s;
 #endif
 #ifdef EDIT_DISTANCE
 	case 'G':
-	    fuzzyGet = upordown;
+	    fuzzyGet = upordown && (actfp || mode == 'i');
 	    break;
 #endif        
 	case 'H':
@@ -488,6 +492,12 @@ register char *s;
 	    notincl("-v");
 #endif
 	    break;
+	case 'V':
+	    trn_version();
+	    putchar('\n');
+	    if (mode == 'i')
+		exit(0);
+	    break;
 	case 'x':
 	    s++;
 	    if (*s == '=') s++;
@@ -529,16 +539,20 @@ register char *s;
 	case 'Z':
 	    s++;
 	    if (*s == '=') s++;
-	    try_mt = FALSE;
-	    try_ov = FALSE;
+	    if (try_mt > 0)
+		try_mt = 0;
+	    if (try_ov > 0)
+		try_ov = 0;
 	    if (upordown) {
 		while (*s) {
 		    switch (*s++) {
 		    case 'o':
-			try_ov = TRUE;
+			if (try_ov == 0)
+			    try_ov = ((mode == 'i' || ov_init())? 1 : -1);
 			break;
 		    case 't':
-			try_mt = TRUE;
+			if (try_mt == 0)
+			    try_mt = ((mode == 'i' || mt_init())? 1 : -1);
 			break;
 		    }
 		}
@@ -585,7 +599,9 @@ pr_switches()
 #ifdef INNERSEARCH
     printf("-g%d ", gline+1);
 #endif
+#ifdef EDIT_DISTANCE
     printf("%cG", mp[fuzzyGet]);
+#endif
     putchar('\n');
 #ifdef VERBOSE
     if (verbose) {
@@ -662,11 +678,11 @@ pr_switches()
 	printf("-z%ld ",(long)actFetchTime / 60);
     else
 	printf("+z ");
-    if (try_mt || try_ov) {
+    if (try_mt > 0 || try_ov > 0) {
 	printf("-Z");
-	if (try_ov)
+	if (try_ov > 0)
 	    putchar('o');
-	if (try_mt)
+	if (try_mt > 0)
 	    putchar('t');
 	putchar(' ');
     } else
